# !/usr/bin/env python
__author__ = "Danelle Cline, Duane Edgington"
__copyright__ = "Copyright 2022, MBARI"
__credits__ = ["MBARI"]
__license__ = "GPL"
__maintainer__ = "Duane Edgington"
__email__ = "duane at mbari.org"
__doc__ = '''

Configuration helper to setup defaults and fetch cloud configuration

@author: __author__
@status: __status__
@license: __license__
'''

import boto3
from configparser import ConfigParser
import datetime as dt
import os
from botocore.exceptions import ClientError
from pathlib import Path
from typing import List

default_training_prefix = 'training'
default_config_ini = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini')

class Config:

    def __init__(self, path: str = None, quiet: bool = False):
        """
        Read the .ini file and parse it
        """
        self.parser = ConfigParser()
        if path:
            self.path = path
        else:
            self.path = default_config_ini

        if not os.path.isfile(self.path):
            raise Exception(f'Bad path to {self.path}. Is your {self.path} missing?')

        self.parser.read(self.path)
        lines = open(self.path).readlines()
        if not quiet:
            print(f"===============>Config file {self.path}<=================")
            for l in lines:
                print(l.strip())

            if not path:
                print(f"============You can override these settings by creating a customconfig.ini file and pass that in with --config=customconfig.ini =====")

    def __call__(self, *args, **kwargs):
        assert len(args) == 2
        return self.parser.get(args[0], args[1])

    def save(self, *args, **kwargs):
        assert len(args) == 3
        self.parser.set(section=args[0], option=args[1], value=args[2])
        with open(self.path, 'w') as fp:
            self.parser.write(fp)

    def get_role(self) -> str:
        """
        Get the user role; first check the environment variable, then the config
        :return role ARN string
        """
        if 'SAGEMAKER_ROLE' in os.environ:
            return os.environ['SAGEMAKER_ROLE']

        sagemaker_arn = self.__call__('aws', 'sagemaker_arn')
        if not sagemaker_arn:
            raise Exception('Run deepsea-ai setup or set the SAGEMAKER_ROLE environment variable')
        return sagemaker_arn

    staticmethod
    def get_account(self) -> str:
        """
        Get the account number associated with this user
        :return:
        """
        account_number = boto3.client('sts').get_caller_identity()['Account']
        print(f'Found account {account_number}')
        return account_number

    staticmethod
    def get_region(self) -> str:
        """
        Get the region associated with this user
        :return:
        """
        session = boto3.session.Session()
        region = session.region_name
        print(f'Found region {region}')
        return region

    staticmethod
    def get_username(self) -> str:
        """
        Get the user name using IAM; if IAM is not configured, this will default to the root user which may be the case
        for a new AWS account
        :return:
        """
        try:
            sts = boto3.client('sts')
            response = sts.get_caller_identity()
            user_name = response['Arn'].split("/")[-1].split("@")[0]
        except ClientError as e:
            # The user_name may be specified in the Access Denied message...
            user_name = "Unknown"

        return user_name


    staticmethod
    def get_tags(self, description: str) -> dict:
        """
        Configure tag dictionary to associate with any AWS resource.
        This is useful for cost accounting and general reporting
        :param description: descriptive information about the use
        :return dictionary with tag key/value pairs
        """
        organization = self.__call__('tags', 'organization')
        project_number = self.__call__('tags', 'project_number')
        stage = self.__call__('tags', 'stage')
        application = self.__call__('tags', 'application')
        user_name = self.get_username()
        deletion_date = (dt.datetime.utcnow() + dt.timedelta(days=90)).strftime('%Y%m%dT%H%M%SZ')
        tag_dict = [{'Key': f'{organization}:project-number', 'Value': project_number},
                    {'Key': f'{organization}:owner', 'Value': user_name},
                    {'Key': f'{organization}:description', 'Value': description},
                    {'Key': f'{organization}:stage', 'Value': stage},
                    {'Key': f'{organization}:application', 'Value': application},
                    {'Key': f'{organization}:deletion-date', 'Value': deletion_date},
                    {'Key': f'{organization}:created-by', 'Value': user_name}]
        return tag_dict

    staticmethod
    def get_resources(self, stack_name: str) -> dict:
        """
        Get resources relevant to the pipeline from the stack name; see deepsea-ai/cluster/stacks
        :param stack_name: name of the stack to query in the ECS cluster
        :return: dictionary with resource names
        """
        client_cf = boto3.client('cloudformation')
        client_ecs = boto3.client('ecs')

        try:
            stack_resources = client_cf.list_stack_resources(StackName=stack_name)
            resources = {'CLUSTER': stack_name}

            # fetch the PROCESSOR environment variable for the single task in the stack; the job is keyed uniquely to it
            key = ['PROCESSOR', 'TRACK_QUEUE', 'VIDEO_QUEUE', 'DEAD_QUEUE', 'TRACK_BUCKET', 'VIDEO_BUCKET']
            for r in stack_resources['StackResourceSummaries']:
                if 'AWS::ECS::TaskDefinition' in r['ResourceType']:
                    arn = r['PhysicalResourceId']
                    task_def = client_ecs.describe_task_definition(taskDefinition=arn)
                    environment = task_def['taskDefinition']['containerDefinitions'][0]['environment']
                    for e in environment:
                        for k in key:
                            if k in e['name']:
                                resources[k] = e['value']
                    break
                if 'AWS::AutoScaling::AutoScalingGroup' in r['ResourceType']:
                    resources['ASG'] = r['PhysicalResourceId']
            return resources
        except ClientError as ex:
            print(ex)
        return None

    staticmethod
    def check_videos(self, input_path: Path) -> List[Path]:
        """
         Check for videos with acceptable suffixes and return the Paths to them
        :param input_path: input path to search (non-recursively)
        :return:
        """
        vid_formats = ['.mov', '.avi', '.mp4', '.mpg', '.mpeg', '.m4v', '.wmv', '.mkv']  # acceptable video suffixes
        files = sorted(input_path.glob('**/*'))
        videos = [x for x in files if x.suffix.lower() in vid_formats and '._' not in x.name]
        num_videos = len(videos)
        assert (num_videos > 0), "No videos to process"
        video_paths = [Path(x) for x in videos]
        return video_paths



#!/usr/bin/env python
###########################################################################################################
# Credit to Alex for his blog: https://alexwlchan.net/2020/11/copying-images-from-docker-hub-to-amazon-ecr/
# and code detailing how to mirror repositories below
###########################################################################################################
import base64
import json
import subprocess

import boto3
from botocore.exceptions import ClientError
from deepsea_ai.config import config as cfg


def get_ecr_repo_names_in_account(ecr_client, *, account_id):
    """
    Returns a set of all the ECR repository names in an AWS account.
    """
    repo_names = set()

    paginator = ecr_client.get_paginator("describe_repositories")
    for page in paginator.paginate(registryId=account_id):
        for repo in page["repositories"]:
            repo_names.add(repo["repositoryName"])

    return repo_names


def docker_login_to_ecr(ecr_client, *, account_id):
    """
    Authenticate Docker against the ECR repository in a particular account.

    The authorization token obtained from ECR is good for twelve hours, so this
    function is cached to save repeatedly getting a token and running `docker login`
    in quick succession.
    """
    response = ecr_client.get_authorization_token(registryIds=[account_id])

    try:
        auth = response["authorizationData"][0]
    except (IndexError, KeyError):
        raise RuntimeError("Unable to get authorization token from ECR!")

    auth_token = base64.b64decode(auth["authorizationToken"]).decode()
    username, password = auth_token.split(":")

    cmd = [
        "docker",
        "login",
        "--username",
        username,
        "--password",
        password,
        auth["proxyEndpoint"],
    ]

    subprocess.check_call(cmd)


def create_ecr_repository(ecr_client, *, name):
    """
    Create a new ECR repository.
    """
    try:
        ecr_client.create_repository(repositoryName=name)
    except ClientError as err:
        if err.response["Error"]["Code"] == "RepositoryAlreadyExistsException":
            pass
        else:
            raise


def docker(*args):
    """
    Shell out to the Docker CLI.
    """
    subprocess.check_call(["docker"] + list(args))


def mirror_docker_hub_images_to_ecr(ecr_client, *, account_id, region, image_tags):
    """
    Given the name/tag of images in Docker Hub, mirror those images to ECR.
    """

    print(f"Creating all ECR repositories in account {account_id}...")
    existing_repos = get_ecr_repo_names_in_account(ecr_client, account_id=account_id)

    mirrored_repos = set(tag.split(":")[0] for tag in image_tags)
    missing_repos = mirrored_repos - existing_repos

    for repo_name in missing_repos:
        ecr_client.create_repository(repositoryName=repo_name)

    print(f"Authenticating Docker with ECR for account {account_id}...")
    docker_login_to_ecr(ecr_client, account_id=account_id)

    for hub_tag in image_tags:
        ecr_tag = f"{account_id}.dkr.ecr.{region}.amazonaws.com/{hub_tag}"
        print(f"Mirroring {hub_tag} to {ecr_tag}")
        docker("pull", hub_tag)
        docker("tag", hub_tag, ecr_tag)
        docker("push", ecr_tag)


def create_role(account_id: str):
    """
    Sets up the IAM role called DeepSeaAI to support ECR and SageMaker
    :param account_id: AWS account ID
    """
    session = boto3.session.Session()
    iam = session.client('iam')

    path = '/'
    role_name = 'DeepSeaAI'
    description = 'DeepSeaAI Role'
    session_secs = 43200

    ASSUME_ROLE_POLICY_JSON = {
        "Version": "2012-10-17",
        "Statement": [
            {
                "Sid": "",
                "Effect": "Allow",
                "Principal": {
                    "Service": "ecs-tasks.amazonaws.com"
                },
                "Action": "sts:AssumeRole"
            },
            {
                "Sid": "",
                "Effect": "Allow",
                "Principal": {
                    "Service": "sagemaker.amazonaws.com"
                },
                "Action": "sts:AssumeRole"
            },
            {
                "Sid": "TrustPolicyStatementThatAllowsEC2ServiceToAssumeTheAttachedRole",
                "Effect": "Allow",
                "Principal": {
                    "Service": "ec2.amazonaws.com"
                },
                "Action": "sts:AssumeRole"
            }
        ]
    }

    ROLE_PERMISSIONS_JSON =  {
            "Version": "2012-10-17",
            "Statement": [{
                "Effect": "Allow",
                "Action": [
                    "iam:GetRole",
                    "iam:PassRole"
                ],
                "Resource": f"arn:aws:iam::{account_id}:role/{role_name}"
            }]
    }

    try:

        policy = f"{role_name}GetAndPassRolePolicy"

        iam = session.client('iam')

        iam.create_policy(
            PolicyName=policy,
            PolicyDocument=json.dumps(ROLE_PERMISSIONS_JSON)
        )

        iam.create_role(
            Path=path,
            RoleName=role_name,
            AssumeRolePolicyDocument=json.dumps(ASSUME_ROLE_POLICY_JSON),
            Description=description,
            MaxSessionDuration=session_secs
        )

        iam.attach_role_policy(
            PolicyArn='arn:aws:iam::aws:policy/AmazonSageMakerFullAccess',
            RoleName=role_name
        )
        iam.attach_role_policy(
            PolicyArn='arn:aws:iam::aws:policy/AmazonS3FullAccess',
            RoleName=role_name
        )
        iam.attach_role_policy(
            PolicyArn=f"arn:aws:iam::aws:policy/{policy}",
            RoleName=role_name
        )

    except ClientError as err:
        if err.response["Error"]["Code"] == "EntityAlreadyExists":
            print(f"Role {role_name} already exists.")
            pass
        else:
            raise


def store_role(config: cfg):
    """
    Save the role in the config
    """
    session = boto3.session.Session()
    iam = session.client('iam')
    results = iam.get_role(RoleName="DeepSeaAI")
    role_arn = results['Role']['Arn']
    print(f'Setting SageMaker Role ARN to {role_arn} in {config.path}')
    config.save('aws', 'sagemaker_arn', role_arn)


if __name__ == "__main__":
    default_config = cfg.Config()
    default_config_ini = cfg.default_config_ini
    account = default_config.get_account()
    region = default_config.get_region()
    image_cfg = ['yolov5_ecr', 'deepsort_ecr', 'strongsort_ecr']
    image_tags = [default_config('aws', t) for t in image_cfg]
    mirror_docker_hub_images_to_ecr(ecr_client=boto3.client("ecr"), account_id=account, region=region, image_tags=image_tags)
    create_role(account_id=account)
    store_role(default_config)




# !/usr/bin/env python
__author__ = "Danelle Cline, Duane Edgington"
__copyright__ = "Copyright 2022, MBARI"
__credits__ = ["MBARI"]
__license__ = "GPL"
__maintainer__ = "Duane Edgington"
__email__ = "duane at mbari.org"
__doc__ = '''

Various commands for monitoring the status of processing or training jobs

@author: __author__
@status: __status__
@license: __license__
'''

import boto3


def print_scaling_activities(resources: dict, num_records: int = 10):
    """
    Print scaling activities for a given resource
    """
    client = boto3.client('autoscaling')
    response = client.describe_scaling_activities(
        ActivityIds=[],
        AutoScalingGroupName=resources['ASG'],
        IncludeDeletedGroups=False,
        MaxRecords=num_records)

    for i in response['Activities']:
        print(f'{i["StartTime"]} {i["Description"]}  {i["Cause"]}')



# !/usr/bin/env python
__author__ = "Danelle Cline, Kevin Barnard"
__copyright__ = "Copyright 2022, MBARI"
__credits__ = ["MBARI"]
__license__ = "GPL"
__maintainer__ = "Danelle Cline"
__email__ = "dcline at mbari.org"
__doc__ = '''

Deepsea-ai database connection and query largely based on the boxjelly api connector

@author: __author__
@status: __status__
@license: __license__
'''
from typing import Optional

import requests

class GraphQLError(requests.HTTPError):
    """
    Error raised when a GraphQL query fails.
    """

    @property
    def message(self):
        """
        Return the error message.
        """
        try:
            response_json = self.response.json()
            message_str = response_json['error']['message']
            return message_str
        except (requests.JSONDecodeError, KeyError):
            return self.response.text

class DeepSeaAIClient:
    """
    Deep Sea AI GraphQL API client.
    """

    def __init__(self, url: str):
        """
        Initialize DeepSeaAI API client.
        """
        self._url = None
        self.url = url

        self._session = requests.Session()

    @property
    def url(self) -> str:
        return self._url

    @url.setter
    def url(self, url: str):
        self._url = url.rstrip('/')

    def execute(self, query: str, **variables):
        """
        Execute a GraphQL query.
        """
        data = {
            'query': query,
            'variables': variables,
        }

        response = self._session.post(
            self.url,
            json=data,
        )

        try:
            response.raise_for_status()
        except requests.HTTPError as e:
            raise GraphQLError(e) from e

        return response.json()



# !/usr/bin/env python
__author__ = "Danelle Cline"
__copyright__ = "Copyright 2022, MBARI"
__credits__ = ["MBARI"]
__license__ = "GPL"
__maintainer__ = "Duane Edgington"
__email__ = "duane at mbari.org"
__doc__ = '''

Train a YOLOv5 model

@author: __author__
@status: __status__
@license: __license__
'''

import boto3
import random
from tqdm import tqdm
import sagemaker
import shutil
import tempfile
import tarfile
from pathlib import Path
from urllib.parse import urlparse
from sagemaker.estimator import Estimator
from deepsea_ai.config import config as cfg


models = ['yolov5n', 'yolov5s', 'yolov5m', 'yolov5l', 'yolov5x', 'yolov5n6', 'yolov5s6', 'yolov5m6', 'yolov5l6',
          'yolov5x6']

def yolov5(data: [Path], input_s3: tuple, ckpts_s3: tuple, model_s3: tuple, epochs:int, batch_size:int,
           volume_size_gb:int,  model:str, instance_type:str, custom_config:cfg.Config):
    """
    Train a YOLOv5 model with SageMaker
    :param data: three part array with compressed artifacts in the order: 0: images, 1: labels, and 2: a text file with the names to ID mapping
    :param input_s3: Input s3 urlparse tuple to the training data
    :param ckpts_s3: Output s3 urlparse tuple to the model checkpoints
    :param model_s3: Output s3 urlparse tuple to the model
    :param epochs: Number of epochs to train; must be > 0
    :param batch_size: Batch size to use; must be > 0
    :param volume_size_gb: Size in GB of the volume to attach to training
    :param model: YOLOv5 model type
    :param instance_type: Type of the AWS instance used, e.g. ml.p2.xlarge
    :param custom_config: configuration
    """
    sagemaker_session = sagemaker.Session()

    # if you are running this outside of a SageMaker notebook, you must set SAGEMAKER_ROLE
    role = custom_config.get_role()

    tags = custom_config.get_tags(f'YOLOv5 training batch {batch_size} model {model}')

    # capture the common metrics
    metric_definitions = [{'Name': 'box_loss', 'Regex': 'box_loss = ([0-9.]+)'},
                          {'Name': 'cls_loss', 'Regex': 'cls_loss = ([0-9.]+)'},
                          {'Name': 'AP50', 'Regex': 'AP50 = ([0-9.]+)'}]

    # add a metric definition for each concept to track - these will be reported during training
    labels = []
    with open(data[2]) as label_file:
        labels = [l.strip() for l in label_file.readlines()]

    assert(len(labels) > 0)

    # can only track up to 40 metrics including the common ones above and some overhead metrics
    for i, name in enumerate(labels):
        if i < 37:
            print(f"Monitoring metric 'AP_{name}'")
            metric_definitions.append({'Name': f'AP_{name}',  'Regex': f'AP_\/{name} = ([0-9.]+)'})
        else:
            print(f"40 limit exceeded for metric monitoring in SageMaker. Skipping over monitoring metric 'AP_{name}'")

    # setup the estimator
    print(f"Saving checkpoints to s3://{ckpts_s3.netloc}/{ckpts_s3.path.lstrip('/')}")
    print(f"Saving model to s3://{model_s3.netloc}/{model_s3.path.lstrip('/')}")

    if model not in models:
        raise Exception(f'Model {model} invalid. Choose a model from {models}')

    img_size = 640
    if '6' in model: img_size = 1280  # all larger (1280x1280) models have the number 6 in them, e.g. yolov5n6
    image_uri = f"{custom_config.get_account()}.dkr.ecr.{custom_config.get_region()}.amazonaws.com/{custom_config('aws', 'yolov5_ecr')}"
    user_name = custom_config.get_username()
    output_ckpts_s3 = f"s3://{ckpts_s3.netloc}/{ckpts_s3.path.lstrip('/')}"
    output_model_s3 = f"s3://{model_s3.netloc}/{model_s3.path.lstrip('/')}"
    training_s3 = f"s3://{input_s3.netloc}/{input_s3.path.lstrip('/')}"
    estimator = Estimator(base_job_name=f'{model}-{user_name}',
                           role=role,
                           tags=tags,
                           image_uri=image_uri,
                           volume_size = volume_size_gb,
    #                        max_wait=43200,
    #                        max_run=42300,
                           enable_sagemaker_metrics=True,
                           checkpoint_s3_uri=output_ckpts_s3,
                           output_path=output_model_s3,
                           instance_count=1,
                           instance_type=instance_type,
                           sagemaker_session=sagemaker_session,
                           input_mode= 'File',
                           metric_definitions=metric_definitions,
                           hyperparameters={
                               'num-epochs': epochs,
                               'model': model,
                               'img-size': img_size,
                               'batch-size': batch_size,
                               'lr': .08,
                               'images': f'/opt/ml/input/data/training/{data[0].name}',
                               'labels': f'/opt/ml/input/data/training/{data[1].name}',
                               'label-map': f'/opt/ml/input/data/training/{data[2].name}'
                            })

    training_data = sagemaker.inputs.TrainingInput(training_s3, distribution='FullyReplicated', content_type='text/plain', s3_data_type='S3Prefix')

    # launch the training then clean-up the checkpoint bucket
    estimator.fit(inputs={'training': training_data})

    print(f'Checkpoints saved in {output_ckpts_s3}')
    s3 = boto3.resource('s3')
    bb = s3.Bucket(model_s3.netloc)
    for obj in bb.objects.filter(Prefix=model_s3.path.lstrip('/')):
        if 'model.tar.gz' in obj.key:
            print(f'Model saved in {output_model_s3}')
            break

def package(bucket_s3: tuple):
    """
    :Package a YOLOv5 model into a format that the deepsea-ai can use in its pipelines.
    :param bucket_s3: Urlparse tuple to the model checkpoints; packages up the best.pt checkpoint plus other config files
    :return:
    """

    s3 = boto3.client('s3')
    s3_resource = boto3.resource('s3')
    b = s3_resource.Bucket(bucket_s3.netloc)

    with tempfile.TemporaryDirectory() as in_tmp_dir:
        in_tmp_path = Path(in_tmp_dir) / 'in'
        in_tmp_path.mkdir()
        out_tmp_path = Path(in_tmp_dir) / 'out'
        out_tmp_path.mkdir()
        out_tar_path = Path(in_tmp_dir) / 'model.tar.gz'

        # download all .yaml and best.pt checkpoints
        for obj in b.objects.filter(Prefix=bucket_s3.path.lstrip('/')):
            if 'best.pt' in obj.key:
                best_checkpoint = out_tmp_path / 'best.pt'
                print(f'Dowloading {obj.key} to {best_checkpoint.as_posix()}...')
                s3.download_file(
                    bucket_s3.netloc,
                    obj.key,
                    best_checkpoint.as_posix()
                )
            if '.yaml' in obj.key:
                yaml = out_tmp_path / Path(obj.key).name
                print(f'Dowloading {obj.key} to {yaml.as_posix()}...')
                s3.download_file(
                    bucket_s3.netloc,
                    obj.key,
                    yaml.as_posix()
                )

        # compress into a model package
        print(f'Creating {out_tar_path}')
        with tarfile.open(out_tar_path.as_posix(), "w:gz") as tar:
            tar.add(out_tmp_path.as_posix())

        # upload to the bucket
        try:
            with open(out_tar_path.as_posix(), "rb") as f:
                target = f"{bucket_s3.path.rstrip('/')}/{out_tar_path.name}"
                print(f'Uploading {out_tar_path} to s3://{bucket_s3.netloc}/{target}...')
                s3.upload_fileobj(f, bucket_s3.netloc, target)
        except:
            print("error in uploading to s3")

        print(f'Done. s3://{bucket_s3.path}/{out_tar_path.name} is now ready to use in a deepsea-ai detection and tracking pipeline')

def split(input_path:Path, output_path:Path):
    #########################################
    # Credit to http://github.com/ultralytics/yolov5 code for this snippet
    #########################################
    def autosplit(path:Path, weights:tuple, annotated_only:bool):
        files = sorted(x for x in path.rglob('*.*') if x.suffix[1:].lower() in ['bmp', 'dng', 'jpeg', 'jpg', 'mpo', 'png', 'tif', 'tiff', 'webp'])  # image files only
        n = len(files)
        random.seed(0)  # for reproducibility
        indices = random.choices([0, 1, 2], weights=weights, k=n)  # assign each image to a split
        txt = ['autosplit_train.txt', 'autosplit_val.txt', 'autosplit_test.txt']
        # remove existing
        for x in txt:
            if (path.parent / x).exists():
                (path.parent / x).unlink()
        print(f'Autosplitting images from {path}' + ', using *.txt labeled images only' * annotated_only)
        for i, img in tqdm(zip(indices, files), total=n):
            if not annotated_only or Path(img2label_paths([str(img)])[0]).exists():  # check label
                with open(path.parent / txt[i], 'a') as f:
                    f.write('./' + img.relative_to(path.parent).as_posix() + '\n')  # add image to txt file
    #########################################

    # do the work in a temporary directory
    with tempfile.TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)
        autosplit(path=input_path, weights=(0.85, 0.10, 0.05), annotated_only=False)

        for v in ['train', 'val', 'test']:
            image_path = temp_path / 'images' / v
            label_path = temp_path / 'labels' / v
            image_path.mkdir(parents=True)
            label_path.mkdir(parents=True)
            split_path = (input_path.parent / f'autosplit_{v}.txt')
            print(f'Splitting {split_path}')
            with (split_path).open('r+t') as f:
                for line in tqdm(f):
                    filename = Path(line.strip())
                    shutil.copy2((input_path / 'labels' / f'{filename.stem}.txt').as_posix(),
                                 (label_path / f'{filename.stem}.txt').as_posix())
                    shutil.copy2((input_path.parent / filename).as_posix(),
                                 (image_path / filename.name).as_posix())

        print(f"Creating {(output_path / 'labels.tar.gz').as_posix()}...")
        with tarfile.open((output_path / 'labels.tar.gz').as_posix(), 'w') as t:
            t.add(label_path.parent, arcname='labels')

        print(f"Creating {(output_path / 'images.tar.gz').as_posix()}...")
        with tarfile.open((output_path / 'images.tar.gz').as_posix(), 'w') as t:
            t.add(image_path.parent, arcname='images')

        print('Done')
if __name__ == '__main__':
    package(urlparse('s3://902005-checkpoints-dev/20220821T005204Z/'))
    split(Path.cwd().parent / 'data' / 'training', Path.cwd().parent / 'data')




# !/usr/bin/env python
__author__ = "Danelle Cline, Duane Edgington"
__copyright__ = "Copyright 2022, MBARI"
__credits__ = ["MBARI"]
__license__ = "GPL"
__maintainer__ = "Duane Edgington"
__email__ = "duane at mbari.org"
__doc__ = '''

Process a collection of videos; assumes videos have previously been uploaded with the upload command

@author: __author__
@status: __status__
@license: __license__
'''

import os
import inspect
import boto3
import json
from datetime import datetime
from pathlib import Path
from deepsea_ai.config import config as cfg
from deepsea_ai.commands.upload_tag import get_prefix

from sagemaker.processing import ScriptProcessor, ProcessingInput, ProcessingOutput

code_path = Path(os.path.abspath(inspect.getfile(inspect.currentframe())))

def script_processor_run(input_s3: tuple, output_s3: tuple, model_s3: tuple, model_size: int,
                         reid_model_url:str, volume_size_gb:int, instance_type:str,
                         config_s3: str, save_vid: bool, conf_thres: float, iou_thres: float,
                         tracker:str, custom_config:cfg.Config, tags:dict):
    """
    Process a collection of videos with the ScriptProcessor
    """
    user_name = custom_config.get_username()
    if tracker not in ['deepsort', 'strongsort']:
        raise Exception(f'{tracker} not currently supported')

    ## TODO: check of config_s3 is a valid s3 bucket with a valid object
    arguments = ['dettrack',
                 f'--conf-thres={conf_thres}',
                 f'--iou-thres={iou_thres}',
                 f'--model-size={model_size}',
                 f"--model-s3=s3://{model_s3.netloc}/{model_s3.path.lstrip('/')}",
                 ]
    if config_s3:
        arguments.append(f'--config-s3={config_s3}')
    if reid_model_url and tracker == 'strongsort': # only support with strongsort as of 11-18-2022
        arguments.append(f'--reid-weights={reid_model_url}')
    else:
        if tracker == 'deepsort':
            arguments.append(f"--config-s3={custom_config('aws','deepsort_track_config_s3')}")
        if tracker == 'strongsort':
            arguments.append(f"--config-s3={custom_config('aws','strongsort_track_config_s3')}")
    if save_vid:
        arguments.append('--save-vid')
    print(arguments)

    # Construct the uri from the config, e.g.
    # mbari/deepsea-yolov5:1.1.2 => 872338704006.dkr.ecr.us-west-2.amazonaws.com/deepsea-yolov5:1.1.2
    account = custom_config.get_account()
    region = custom_config.get_region()
    image_uri_docker = {'deepsort':  custom_config('aws', 'deepsort_ecr'), 'strongsort': custom_config('aws', 'strongsort_ecr')}
    image_uri_ecr = f"{account}.dkr.ecr.{region}.amazonaws.com/{image_uri_docker[tracker]}"

    script_processor = ScriptProcessor(command=['python3'],
                                       image_uri=image_uri_ecr,
                                       role=custom_config.get_role(),
                                       instance_count=1,
                                       base_job_name=f'{tracker}-yolov5-{user_name}',
                                       instance_type=instance_type,
                                       volume_size_in_gb=volume_size_gb,
                                       max_runtime_in_seconds=172800,
                                       tags=tags)
    script_processor.run(code=f'{code_path.parent.parent.parent}/deepsea_ai/pipeline/run_{tracker}.py',
                         arguments=arguments,
                         inputs=[ProcessingInput(
                             source=f"s3://{input_s3.netloc}/{input_s3.path.lstrip('/')}",
                             destination='/opt/ml/processing/input')],
                         outputs=[ProcessingOutput(source='/opt/ml/processing/output',
                                                   destination=f"s3://{output_s3.netloc}/{output_s3.path.lstrip('/')}")]
                         )


def batch_run(resources: dict, video_path: Path, job_name: str, user_name: str, clean: bool, conf_thres: float, iou_thres: float):
    """
    Process a collection of videos in with a cluster in the Elastic Container Service [ECS]
    """
    # the queue to submit the processing message to
    queue_name = resources['VIDEO_QUEUE']

    # Get the service resource
    sqs = boto3.resource('sqs')

    prefix_path = get_prefix(video_path)

    # Get the queue
    queue = sqs.get_queue_by_name(QueueName=queue_name)
    message_dict = {"video": f"{prefix_path}/{video_path.name}",
                    "clean": "True" if clean else "False",
                    "user_name": user_name,
                    "job_name": job_name,
                    "conf_thres": conf_thres,
                    "iou_thres": iou_thres}
    json_object = json.dumps(message_dict, indent=4)

    now = datetime.utcnow()

    # create a message group based on the time; somewhat arbitrary; maybe refine to the hour to avoid collisions
    # from multiple users submitting the same kind of job
    group_id = now.strftime("%Y%m%dT%H%M%SZ")

    # create a new message
    response = queue.send_message(MessageBody=json_object, MessageGroupId=resources['CLUSTER'] + f"{group_id}")
    print(f"Message queued to {queue_name}. MessageId: {response.get('MessageId')}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['deepsea_ai',
 'deepsea_ai.commands',
 'deepsea_ai.config',
 'deepsea_ai.database',
 'deepsea_ai.pipeline',
 'deepsea_ai.tests']

package_data = \
{'': ['*']}

install_requires = \
['awscli>=1.25.71,<2.0.0',
 'boto3>=1.24.70,<2.0.0',
 'click>=8.0.0,<9.0.0',
 'requests>=2.26.0,<3.0.0',
 'sagemaker>=2.102.0,<3.0.0',
 'tqdm>=4.41.0,<5.0.0']

entry_points = \
{'console_scripts': ['deepsea-ai = deepsea_ai.__main__:cli']}

setup_kwargs = {
    'name': 'deepsea-ai',
    'version': '1.17.1',
    'description': 'DeepSeaAI is a Python package to simplify processing deep sea video in AWS from a command line.',
    'long_description': '[![MBARI](https://www.mbari.org/wp-content/uploads/2014/11/logo-mbari-3b.png)](http://www.mbari.org)\n[![semantic-release](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--release-e10079.svg)](https://github.com/semantic-release/semantic-release)\n![license-GPL](https://img.shields.io/badge/license-GPL-blue)\n[![Python](https://img.shields.io/badge/language-Python-blue.svg)](https://www.python.org/downloads/)\n\n**DeepSeaAI** is a Python package to simplify processing deep sea video in [AWS](https://aws.amazon.com) from a command line. \n \nIt includes reasonable defaults that have been optimized for deep sea video. The goal is to simplify running these algorithms in AWS.\n\nDeepSea-AI currently supports:\n\n - *Training [YOLOv5](http://github.com/ultralytics/yolov5) object detection* models\n - *Processing video with [YOLOv5](http://github.com/ultralytics/yolov5) detection and tracking pipelines* using either:\n     * [DeepSort](https://github.com/mikel-brostrom/Yolov5_DeepSort_Pytorch) tracking\n     * [StrongSort](https://github.com/mikel-brostrom/Yolov5_StrongSORT_OSNet) tracking\n\n## Install\n\nSetup [an AWS account](https://aws.amazon.com).\n\nAfter you have setup your AWS account, configure it using the awscli tool, and confirm your AWS Account by listing your s3 buckets\n\n```\npip install awscli\naws configure\naws --version\naws s3 ls \n```\n\nInstall and update using [pip](https://pip.pypa.io/en/stable/getting-started/) in a Python>=3.8.0 environment:\n\n```shell\npip install -U deepsea-ai\n```\n\nSetup your AWS account for use with this module with\n\n```shell\ndeepsea-ai setup\n```\n\n\n\n## Tutorials\n\n* [FathomNet](docs/notebooks/fathomnet_train.ipynb) ✨ Recommended first step to learn more about how to train a YOLOv5 object detection model using freely available FathomNet data\n\nThe best way to use the tutorials is with [Anaconda](https://www.anaconda.com/products/distribution).\n\n### Create the Anaconda environment\n\nThis will create an environment called *deepsea-ai-notebooks* and make that available in your local jupyter notebook as the kernel named *deepsea-ai-notebooks*\n```\nconda env create \nconda activate deepsea-ai-notebooks\npip install ipykernel\npython -m ipykernel install --user --name=deepsea-ai-notebooks\n```\n\n### Launch jupyter\n\n```\ncd docs/notebooks\njupyter notebook\n```\n---\n\n## Commands\n\n* `deepsea-ai setup --help` - Setup the AWS environment. Must run this once before any other commands.\n* [`deepsea-ai train --help` - Train a YOLOv5 model and save the model to a bucket](commands/train.md)\n* [`deepsea-ai process --help` - Process one or more videos and save the results to  a bucket](commands/process.md)\n* [`deepsea-ai ecsprocess --help` - Process one or more videos using the Elastic Container Service and save the results to a bucket](commands/process.md)\n* [`deepsea-ai split --help` - Split your training data. This is required before the train command.](data.md) \n* `deepsea-ai -h` - Print help message and exit.\n \nSource code is available at [github.com/mbari-org/deepsea-ai](https://github.com/mbari-org/deepsea-ai/).\n  \nFor more details, see the [official documentation](http://docs.mbari.org/deepsea-ai/install).',
    'author': 'Danelle Cline',
    'author_email': 'dcline@mbari.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/mbari-org/deepsea-ai',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
